using JuMP
using Ipopt
using Plots
##Supplementary Files
include("CollMat.jl")
include("Parameters.jl")
include("InitiStates.jl")
#choose backend for plots
# plotlyjs()

##Define the optimizer Function
function Plant_optimizer(u0, t_min, t_min_inp, Nw, Inits, αₚₒ, α_gl, γ, Plotting)
    #Required Parameters
    Nx = 3;
    Nz = 12;
    Nu = 1;
    NCP = 4
    # NFE = round(Int, t_min/NCP)
    NFE = round(Int, t_min * 60/NCP)
    # Nw  =  3;


    # u0=[4.0; 4.0; 4.0]

    # Inits = Initials()
    # @unpack_Initials Inits

    m_ga0    = Inits[:m_ga_pro]
    m_gt0    = Inits[:m_gt_pro]
    m_ot0    = Inits[:m_ot_pro]
    P_a0     = Inits[:P_a0]
    ρ_a0     = Inits[:ρ_a0]
    P_wh0    = Inits[:P_wh0]
    ρ_m0     = Inits[:ρ_m0]
    ω_pc0    = Inits[:ω_pc0]
    P_wi0    = Inits[:P_wi0]
    ω_iv0    = Inits[:ω_iv0]
    ω_pg0    = Inits[:ω_pg0]
    ω_po0    = Inits[:ω_po0]
    P_bh0    = Inits[:P_bh0]
    ω_ro0    = Inits[:ω_ro0]
    ω_rg0    = Inits[:ω_rg0]
    dx0      = Inits[:dx0]
    x0       = Inits[:x0]

    M = Collocation_Matrix(NCP)
    dt = 4;
    # αₚₒ = 1;
    # α_gl = 0.01;
    # γ = 0;

    # Parameter Loading
    if Nw == 1
        par = WellPar1()
        @unpack_WellPar1 par
    elseif Nw == 2
        par = WellPar2()
        @unpack_WellPar2 par
    elseif Nw == 3
        # par = WellPar3()
        # @unpack_WellPar3 par
        par = WellPar3diff()
        @unpack_WellPar3diff par    
    else
        println("Nw must be either 1 or 2 or 3!")
        #More data can be added in Parameters.jl 
    end      

    ##Model Defining
    m1 = Model(Ipopt.Optimizer)
    #Defining the Variables===========================================================
    @variable(m1,  x[1:Nw, 1:Nx, 1:NFE, 1:NCP+1])
    @variable(m1, dx[1:Nw, 1:Nx, 1:NFE, 1:NCP  ])
    @variable(m1,  z[1:Nw, 1:Nz, 1:NFE, 1:NCP  ])
    @variable(m1,  u[1:Nw, 1:Nu, 1:NFE         ])
    #Bounds and Starting Points===========================================================
    for nw in 1:Nw, nx in 1:Nx, nz in 1:Nz, nfe in 1:NFE, ncp in 1:NCP, nu in 1:Nu
        #Bounds
        set_lower_bound(x[nw, nx, nfe, ncp], 0.01)
        set_upper_bound(x[nw, nx , nfe, ncp], 1e7)
        set_lower_bound(z[nw, nz, nfe, ncp], 0)
        set_upper_bound(z[nw, nz , nfe, ncp], 1e7)
        set_lower_bound(u[nw, nu, nfe], 3)
        set_upper_bound(u[nw, nu, nfe], 10)
        
    end
    #Start Values 
    for nw in 1:Nw, nfe in 1:t_min_inp*15, ncp in 1:NCP
        set_start_value(x[nw, 1, nfe, ncp],      m_ga0[nw, nfe])
        set_start_value(x[nw, 2, nfe, ncp],      m_gt0[nw, nfe])
        set_start_value(x[nw, 3, nfe, ncp],      m_ot0[nw, nfe])

        set_start_value(u[nw, 1, nfe],          u0[nw, nfe])
        
        set_start_value(z[nw, 1, nfe, ncp],     P_a0[nw, nfe])
        set_start_value(z[nw, 2, nfe, ncp],     ρ_a0[nw, nfe])
        set_start_value(z[nw, 3, nfe, ncp],     ρ_m0[nw, nfe])
        set_start_value(z[nw, 4, nfe, ncp],     P_wh0[nw, nfe])
        set_start_value(z[nw, 5, nfe, ncp],     P_wi0[nw, nfe])
        set_start_value(z[nw, 6, nfe, ncp],     P_bh0[nw, nfe])
        set_start_value(z[nw, 7, nfe, ncp],     ω_iv0[nw, nfe])
        set_start_value(z[nw, 8, nfe, ncp],     ω_pc0[nw, nfe])
        set_start_value(z[nw, 9, nfe, ncp],     ω_pg0[nw, nfe])
        set_start_value(z[nw, 10, nfe, ncp],    ω_po0[nw, nfe])
        set_start_value(z[nw, 11, nfe, ncp],    ω_ro0[nw, nfe])
        set_start_value(z[nw, 12, nfe, ncp],    ω_rg0[nw, nfe])
    end
    for nw in 1:Nw, nfe in t_min_inp*15 + 1:NFE, ncp in 1:NCP
        set_start_value(x[nw, 1, nfe, ncp],      m_ga0[nw, end])
        set_start_value(x[nw, 2, nfe, ncp],      m_gt0[nw, end])
        set_start_value(x[nw, 3, nfe, ncp],      m_ot0[nw, end])

        set_start_value(u[nw, 1, nfe],          u0[nw, end])

        set_start_value(z[nw, 1, nfe, ncp],     P_a0[nw, end])
        set_start_value(z[nw, 2, nfe, ncp],     ρ_a0[nw, end])
        set_start_value(z[nw, 3, nfe, ncp],     ρ_m0[nw, end])
        set_start_value(z[nw, 4, nfe, ncp],     P_wh0[nw, end])
        set_start_value(z[nw, 5, nfe, ncp],     P_wi0[nw, end])
        set_start_value(z[nw, 6, nfe, ncp],     P_bh0[nw, end])
        set_start_value(z[nw, 7, nfe, ncp],     ω_iv0[nw, end])
        set_start_value(z[nw, 8, nfe, ncp],     ω_pc0[nw, end])
        set_start_value(z[nw, 9, nfe, ncp],     ω_pg0[nw, end])
        set_start_value(z[nw, 10, nfe, ncp],    ω_po0[nw, end])
        set_start_value(z[nw, 11, nfe, ncp],    ω_ro0[nw, end])
        set_start_value(z[nw, 12, nfe, ncp],    ω_rg0[nw, end])
    end



    #Expressions Variables======================================================= 
    #(makes it easier to write DAE Equation) #?add variables and change indices
    @NLexpressions(m1, begin
        m_ga[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP+1],           x[nw, 1, nfe, ncp]      
        m_gt[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP+1],           x[nw, 2, nfe, ncp]       
        m_ot[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP+1],           x[nw, 3, nfe, ncp] 

        dm_ga[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           dx[nw, 1, nfe, ncp]          
        dm_gt[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           dx[nw, 2, nfe, ncp]       
        dm_ot[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           dx[nw, 3, nfe, ncp]      
        
        P_a[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],            z[nw, 1, nfe, ncp]
        ρ_a[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],            z[nw, 2, nfe, ncp]
        ρ_m[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],            z[nw, 3, nfe, ncp]
        P_wh[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 4, nfe, ncp]
        P_wi[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 5, nfe, ncp]
        P_bh[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 6, nfe, ncp]
        ω_iv[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 7, nfe, ncp]
        ω_pc[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 8, nfe, ncp]
        ω_pg[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 9, nfe, ncp]
        ω_po[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 10, nfe, ncp]
        ω_ro[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 11, nfe, ncp]
        ω_rg[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],           z[nw, 12, nfe, ncp]
        
        ω_gl[nw in 1:Nw, nfe in 1:NFE],                          u[nw, 1, nfe]
    end)
    #Set the ODEs==========================================================
    @NLconstraints(m1, begin
        ODE1[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    dm_ga[nw, nfe, ncp]      == (ω_gl[nw, nfe] - ω_iv[nw, nfe, ncp])*1e-3
        ODE2[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    dm_gt[nw, nfe, ncp]      == (ω_iv[nw, nfe, ncp] - ω_pg[nw, nfe, ncp] + ω_rg[nw, nfe, ncp] * 1e-1)*1e-3
        ODE3[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    dm_ot[nw, nfe, ncp]      == (ω_ro[nw, nfe, ncp] - ω_po[nw, nfe, ncp])*1e-3
        #ODEn[nfe in 1:NFE, ncp in 1:NCP],   ...
    end)
    #Algebraic Equations==================================================
    @NLconstraints(m1, begin
    #Defining Model Algebraic Equations in each line
        Constr_Alg1[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    P_a[nw, nfe, ncp]   == 1e-5*(((R[nw]*T_a[nw]/(V_a[nw]*Mw[nw]) + g[nw]*H_a[nw]/V_a[nw])*m_ga[nw, nfe, ncp]*1e3) + (Mw[nw]/(R[nw]*T_a[nw])*((R[nw]*T_a[nw]/(V_a[nw]*Mw[nw]) + g[nw]*H_a[nw]/V_a[nw])*m_ga[nw, nfe, ncp]*1e3))*g[nw]*H_a[nw]) # annulus pressure
        Constr_Alg2[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    ρ_a[nw, nfe, ncp]   == 1e-2*(Mw[nw]/(R[nw]*T_a[nw])*P_a[nw, nfe, ncp]*1e5)  # gas, in annulus
        Constr_Alg3[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    P_wh[nw, nfe, ncp]  == 1e-5*(((R[nw]*T_w[nw]/Mw[nw])*(m_gt[nw, nfe, ncp]*1e3/(L_w[nw]*A_w[nw] + L_bh[nw]*A_bh[nw] - m_ot[nw, nfe, ncp]*1e3/ρₒᵢₗ[nw]))) - ((m_gt[nw, nfe, ncp]*1e3+m_ot[nw, nfe, ncp]*1e3 )/(L_w[nw]*A_w[nw]))*g[nw]*H_w[nw]/2) # wellhead pressure
        Constr_Alg4[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    ρ_m[nw, nfe, ncp]   == 1e-2*(((m_gt[nw, nfe, ncp]*1e3 + m_ot[nw, nfe, ncp]*1e3)*P_wh[nw, nfe, ncp]*1e5*Mw[nw]*ρₒᵢₗ[nw])/(m_ot[nw, nfe, ncp]*1e3*P_wh[nw, nfe, ncp]*1e5*Mw[nw] + ρₒᵢₗ[nw]*R[nw]*T_w[nw]*m_gt[nw, nfe, ncp]*1e3))  # mixture, in tubing
        Constr_Alg5[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    ω_pc[nw, nfe, ncp]  == C_pc[nw]*((ρ_m[nw, nfe, ncp]*1e2*(P_wh[nw, nfe, ncp]*1e5 - P_m[nw]*1e5))^2)^(1/4) # total flow through choke
        Constr_Alg6[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    P_wi[nw, nfe, ncp]  == 1e-5*((P_wh[nw, nfe, ncp]*1e5 + g[nw]/(A_w[nw]*L_w[nw])*(m_ot[nw, nfe, ncp]*1e3+m_gt[nw, nfe, ncp]*1e3-ρₒᵢₗ[nw]*L_bh[nw]*A_bh[nw])*H_w[nw] + 128*mu_oil[nw]*L_w[nw]*ω_pc[nw, nfe, ncp]/(3.141*D_w[nw]^4*((m_gt[nw, nfe, ncp]*1e3 + m_ot[nw, nfe, ncp]*1e3)*P_wh[nw, nfe, ncp]*1e5*Mw[nw]*ρₒᵢₗ[nw])/(m_ot[nw, nfe, ncp]*1e3*P_wh[nw, nfe, ncp]*1e5*Mw[nw] + ρₒᵢₗ[nw]*R[nw]*T_w[nw]*m_gt[nw, nfe, ncp]*1e3)))) 
        Constr_Alg7[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    ω_iv[nw, nfe, ncp]  == C_iv[nw]*((ρ_a[nw, nfe, ncp]*1e2*(P_a[nw, nfe, ncp]*1e5 - P_wi[nw, nfe, ncp]*1e5))^2)^(1/4)
        Constr_Alg8[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    ω_pg[nw, nfe, ncp]  == (m_gt[nw, nfe, ncp]*1e3/(m_gt[nw, nfe, ncp]*1e3+m_ot[nw, nfe, ncp]*1e3))*ω_pc[nw, nfe, ncp] # produced gas rate
        Constr_Alg9[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],    ω_po[nw, nfe, ncp]  == (m_ot[nw, nfe, ncp]*1e3/(m_gt[nw, nfe, ncp]*1e3+m_ot[nw, nfe, ncp]*1e3))*ω_pc[nw, nfe, ncp] # produced oil rate
        Constr_Alg10[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],   P_bh[nw, nfe, ncp]  == 1e-5*(P_wi[nw, nfe, ncp]*1e5 + ρₒᵢₗ[nw]*g[nw]*H_bh[nw] + 128*mu_oil[nw]*L_bh[nw]*ω_po[nw, nfe, ncp]/(3.14*D_bh[nw]^4*ρₒᵢₗ[nw])) 
        Constr_Alg11[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],   ω_ro[nw, nfe, ncp]  == (PI[nw])*1e-6*(Press_r[nw]*1e5 - P_bh[nw, nfe, ncp]*1e5) 
        Constr_Alg12[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],   ω_rg[nw, nfe, ncp]  == 1e1*GOR[nw]*ω_ro[nw, nfe, ncp]
        #Constr_Alg999[nfe=1:NFE, ncp=1:NCP], alg[999,nfe,ncp] ==
        Constr_InEq1[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],   (P_wh[nw, nfe, ncp] - P_m[nw])*1e5 >= 0
        Constr_InEq2[nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP],   (P_a[nw, nfe, ncp] - P_wi[nw, nfe, ncp])*1e5 >= 0  
    end)

    #Collcation!===========================================================
    @NLconstraints(m1, begin
        Coll_Eq_Diff[nw in 1:Nw,     nx in 1:Nx,  nfe in 1:NFE,    ncp in 1:NCP],     x[nw, nx, nfe, ncp+1] == x[nw, nx, nfe, 1] + sum(M[ncp, i] * dt * dx[nw, nx, nfe, i] for i in 1:NCP)
        Cont_Eq_First[nw in 1:Nw,    nx in 1:Nx],                                     x[nw, nx, 1, 1]       == x0[nw, nx]
        Cont_Eq_rest[nw in 1:Nw,     nx in 1:Nx,  nfe in 2:NFE],                      x[nw, nx, nfe, 1]     == x[nw, nx, nfe-1, end]
    end)
    ##constraint on input===========================================================
    @NLconstraints(m1, begin
        #Defining Inequality Constraints in each line
        # Constr_Inp1[nw in 1:Nw], abs((ω_gl[nw, 1] - u0[nw]) / (ω_gl[nw, 1]))       <= 0.5
        # Constr_Inp2[nw in 1:Nw, nfe in 2:NFE], abs((ω_gl[nw, nfe] - ω_gl[nw, nfe-1]) / (ω_gl[nw, nfe]))       <= 1
        #Constr_Inp1[nfe in 1:NFE-1], abs((ω_gl[1, nfe+1] - ω_gl[1, nfe]) / (ω_gl[1,nfe]))       <= 0.5
        #Constr_Inp2[nfe in 1:NFE-1], abs((ω_gl[2, nfe+1] - ω_gl[2, nfe]) / (ω_gl[2,nfe]))       <= 0.5
        #Constr_Inp3[nfe in 1:NFE-1], abs((ω_gl[3, nfe+1] - ω_gl[3, nfe]) / (ω_gl[3,nfe]))       <= 0.5
		#Define constraint on maximum handling

        Constr_InpSum[nfe in 1:NFE, ncp=1:NCP], sum(ω_pg[i, nfe, ncp] for i in 1:Nw)    <=    28
		#Define constraint on maximum gas available

		Constr_InpSum2[nfe in 1:NFE], sum((ω_gl[i, nfe]) for i in 1:Nw)    <=     30

        #In case of more states - pattern
        #Constr_Ineq999[nfe=1:NFE, ncp=1:NCP], alg[999,nfe,ncp] ==
    end)
    ##bjective Function===========================================================
    # @NLobjective(m1, Min,  -(sum(αₚₒ * ω_po[nw, nfe, end] - α_gl * ω_gl[nw, nfe] for nw in 1:Nw, nfe in 1:NFE) - γ * sum((ω_gl[nfe+1] - ω_gl[nfe])^2 for nw in 1:Nw, nfe in 1:NFE-1)))
    @NLobjective(m1, Min ,  -(αₚₒ * sum(ω_po[nw, nfe, ncp] for nw in 1:Nw, nfe in 1:NFE, ncp in 1:NCP) - α_gl * sum(ω_gl[nw, nfe] for nw in 1:Nw, nfe in 1:NFE) -  γ * sum(((ω_gl[nw, nfe+1] - ω_gl[nw, nfe])^2) for nw in 1:Nw, nfe in 1:NFE-1) - γ*sum((ω_gl[nw, 1]-u0[nw,end])^2 for nw in 1:Nw)))
    #@NLobjective(m1, Min,  -sum(αₚₒ * ω_po[nw, nfe, end] - α_gl * ω_gl[nw, nfe] for nw in 1:Nw, nfe in 1:NFE) )

    ##===========================================================
	optimize!(m1)
    JuMP.termination_status(m1)
    JuMP.solve_time(m1)
    ##----------------------------------------------------------
    return JuMP.value.(u)[:,1,1:t_min_inp * 15 + 1], JuMP.value.(ω_po)[:, 1:t_min_inp * 15 + 1, end], JuMP.value.(ω_pg)[:, 1:t_min_inp * 15 + 1, end], objective_value(m1)
    # return JuMP.value.(u)[:,1,:],
    #        JuMP.value.(z)[:, 10, :, end],
    #        JuMP.value.(z)[:, 9, :, end]
end