@with_kw struct WellPar1
    # L, H, D = Length, Height, Diameter [m]
    # well
    L_w = 1500 # length well
    H_w = 1000 # height well
    D_w = 0.121 # diameter well
    # bottom hole
    L_bh = 500 
    H_bh = 500
    D_bh = 0.121
    # annulus
    L_a =  L_w
    H_a = H_w
    D_a = 0.189

    ρₒᵢₗ = 800 # density of oil, kg/m3
    mu_oil = 1*0.001

    C_iv = 1e-3 # injection valve characteristic, m2
    C_pc = 2e-3 # choke valve characteristic, m2

    GOR = 0.1 # Gas Oil Ratio
    Press_r =  150 # reservoir pressure
    PI = 2.2 # Productivity index, kg/s/bar
    P_m = 20  # manifold pressure
    T_a = 28 + 273 # K
    T_w = 32  + 273 # K

    Mw = 20e-3 # ? units kg/mol
    #Other Parameter
    A_bh = π*D_bh^2/4
    LA_bh  = L_bh*A_bh
    A_w = π*D_w^2/4
    LwAw  = L_w*A_w
    g = 9.81 # m/s2
    R = 8.314 # J/mol K
    V_a = L_a*(π*(D_a/2)^2 - π*(D_w/2)^2) 
end

@with_kw struct WellPar2
    # L, H, D = Length, Height, Diameter [m]
    # well
    L_w = 1500 .*ones(2) # length well
    H_w = 1000 .*ones(2) # height well
    D_w = 0.121 .*ones(2) # diameter well
    # bottom hole
    L_bh = 500 .*ones(2) 
    H_bh = 500 .*ones(2)
    D_bh = 0.121 .*ones(2)
    # annulus
    L_a =  L_w .*ones(2)
    H_a = H_w .*ones(2)
    D_a = 0.189 .*ones(2)
    ρₒᵢₗ = 800 .*ones(2) # density of oil, kg/m3
    mu_oil = 1*0.001 .*ones(2)

    C_iv = 1e-3 .*ones(2) # injection valve characteristic, m2
    C_pc = 2e-3 .*ones(2) # choke valve characteristic, m2

    GOR = 0.1 .*ones(2) # Gas Oil Ratio
    Press_r = 150 .*ones(2) # reservoir pressure
    PI = 2.2 .*ones(2) # Productivity index, kg/s/bar
    P_m = 20 .*ones(2) # manifold pressure
    T_a = (28 + 273) .*ones(2) # K
    T_w = (32  + 273) .*ones(2) # K

    Mw = 20e-3 .*ones(2) # ? units kg/mol
    #Other Parameter
    A_bh = (π.*D_bh.^2/4) .*ones(2)
    LA_bh  = (L_bh.*A_bh) .*ones(2)
    A_w = (π.*D_w.^2/4) .*ones(2)
    LwAw  = (L_w.*A_w) .*ones(2)
    g = 9.81 .*ones(2) # m/s2
    R = 8.314 .*ones(2) # J/mol K
    V_a = (L_a.*(π.*(D_a./2).^2 - π.*(D_w./2).^2) ) .*ones(2)
end

@with_kw struct WellPar3
    # L, H, D = Length, Height, Diameter [m]
    # well
    L_w = [1500.0, 1500.0, 1500.0] # length well
    H_w = [1000.0, 1000.0, 1000.0] # height well
    D_w = [0.121, 0.121, 0.121]  # diameter well
    # bottom hole
    L_bh = [500.0, 500.0, 500.0]  
    H_bh = [500.0, 500.0, 500.0]
    D_bh = [0.121, 0.121, 0.121]
    # annulus
    L_a =  [1500.0, 1500.0, 1500.0] 
    H_a = [1000.0, 1000.0, 1000.0]
    D_a = [0.189, 0.189, 0.189] 

    ρₒᵢₗ = [800.0, 800.0, 800.0]   # density of oil, kg/m3
    mu_oil = [0.001, 0.001, 0.001]
    C_iv = [1e-3, 1e-3, 1e-3]  # injection valve characteristic, m2
    C_pc = [2e-3, 2e-3, 2e-3] # choke valve characteristic, m2

    GOR = [0.1, 0.1, 0.1] # Gas Oil Ratio
    Press_r = [150.0, 150.0, 150.0] # reservoir pressure
    PI = [2.2, 2.2, 2.2] # Productivity index, kg/s/bar
    P_m = [20.0, 20.0, 20.0]  # manifold pressure
    T_a = [301.0, 301.0, 301.0]   # K
    T_w = [305.0, 305.0, 305.0]  # K

    Mw = [0.02, 0.02, 0.02]  # Mulecular weight units kg/mol
    #Other Parameter
    A_bh = π.*(D_bh./2).^2
    LA_bh  = L_bh.*A_bh
    A_w = (π.*D_w.^2/4)
    LwAw  = L_w.*A_w
    g = [9.81, 9.81, 9.81] # m/s2
    R = [8.314, 8.314, 8.314] # J/mol K
    V_a = L_a.*(π.*(D_a./2).^2 - π.*(D_w./2).^2)
end

@with_kw struct WellPar3diff
    # L, H, D = Length, Height, Diameter [m]
    # well
    L_w = [1500.0, 1500.0, 1500.0] # length well
    H_w = [1000.0, 1000.0, 1000.0] # height well
    D_w = [0.121, 0.121, 0.121]  # diameter well
    # bottom hole
    L_bh = [500.0, 500.0, 500.0]  
    H_bh = [500.0, 500.0, 500.0]
    D_bh = [0.121, 0.121, 0.121]
    # annulus
    L_a =  [1500.0, 1500.0, 1500.0] 
    H_a = [1000.0, 1000.0, 1000.0]
    D_a = [0.189, 0.189, 0.189] 

    ρₒᵢₗ = [800.0, 800.0, 800.0]   # density of oil, kg/m3
    mu_oil = [0.001, 0.001, 0.001]
    C_iv = [1e-3, 1e-3, 1e-3]  # injection valve characteristic, m2
    C_pc = [2e-3, 2e-3, 2e-3] # choke valve characteristic, m2

    GOR = [0.1, 0.1, 0.1] # Gas Oil Ratio
    Press_r = [150.0, 150.0, 150.0] # reservoir pressure
    PI = [1.2, 2.2, 3.2] # Productivity index, kg/s/bar
    P_m = [20.0, 20.0, 20.0]  # manifold pressure
    T_a = [301.0, 301.0, 301.0]   # K
    T_w = [305.0, 305.0, 305.0]  # K

    Mw = [0.02, 0.02, 0.02]  # Mulecular weight units kg/mol
    #Other Parameter
    A_bh = π.*(D_bh./2).^2
    LA_bh  = L_bh.*A_bh
    A_w = (π.*D_w.^2/4)
    LwAw  = L_w.*A_w
    g = [9.81, 9.81, 9.81] # m/s2
    R = [8.314, 8.314, 8.314] # J/mol K
    V_a = L_a.*(π.*(D_a./2).^2 - π.*(D_w./2).^2)
end

@with_kw struct WellPar3diffp5
    # L, H, D = Length, Height, Diameter [m]
    # well
    L_w = [1500.0, 1500.0, 1500.0] # length well
    H_w = [1000.0, 1000.0, 1000.0] # height well
    D_w = [0.121, 0.121, 0.121]  # diameter well
    # bottom hole
    L_bh = [500.0, 500.0, 500.0]  
    H_bh = [500.0, 500.0, 500.0]
    D_bh = [0.121, 0.121, 0.121]
    # annulus
    L_a =  [1500.0, 1500.0, 1500.0] 
    H_a = [1000.0, 1000.0, 1000.0]
    D_a = [0.189, 0.189, 0.189] 

    ρₒᵢₗ = [800.0, 800.0, 800.0]   # density of oil, kg/m3
    mu_oil = [0.001, 0.001, 0.001]
    C_iv = [1e-3, 1e-3, 1e-3]  # injection valve characteristic, m2
    C_pc = [2e-3, 2e-3, 2e-3] # choke valve characteristic, m2

    GOR = [0.1, 0.1, 0.1] # Gas Oil Ratio
    Press_r = [150.0, 150.0, 150.0] # reservoir pressure
    PI = [1.26, 2.31, 3.36] # Productivity index, kg/s/bar
    P_m = [20.0, 20.0, 20.0]  # manifold pressure
    T_a = [301.0, 301.0, 301.0]   # K
    T_w = [305.0, 305.0, 305.0]  # K

    Mw = [0.02, 0.02, 0.02]  # Mulecular weight units kg/mol
    #Other Parameter
    A_bh = π.*(D_bh./2).^2
    LA_bh  = L_bh.*A_bh
    A_w = (π.*D_w.^2/4)
    LwAw  = L_w.*A_w
    g = [9.81, 9.81, 9.81] # m/s2
    R = [8.314, 8.314, 8.314] # J/mol K
    V_a = L_a.*(π.*(D_a./2).^2 - π.*(D_w./2).^2)
end

@with_kw struct WellPar3diffm5
    # L, H, D = Length, Height, Diameter [m]
    # well
    L_w = [1500.0, 1500.0, 1500.0] # length well
    H_w = [1000.0, 1000.0, 1000.0] # height well
    D_w = [0.121, 0.121, 0.121]  # diameter well
    # bottom hole
    L_bh = [500.0, 500.0, 500.0]  
    H_bh = [500.0, 500.0, 500.0]
    D_bh = [0.121, 0.121, 0.121]
    # annulus
    L_a =  [1500.0, 1500.0, 1500.0] 
    H_a = [1000.0, 1000.0, 1000.0]
    D_a = [0.189, 0.189, 0.189] 

    ρₒᵢₗ = [800.0, 800.0, 800.0]   # density of oil, kg/m3
    mu_oil = [0.001, 0.001, 0.001]
    C_iv = [1e-3, 1e-3, 1e-3]  # injection valve characteristic, m2
    C_pc = [2e-3, 2e-3, 2e-3] # choke valve characteristic, m2

    GOR = [0.1, 0.1, 0.1] # Gas Oil Ratio
    Press_r = [150.0, 150.0, 150.0] # reservoir pressure
    PI = [1.14, 2.09, 3.04] # Productivity index, kg/s/bar
    P_m = [20.0, 20.0, 20.0]  # manifold pressure
    T_a = [301.0, 301.0, 301.0]   # K
    T_w = [305.0, 305.0, 305.0]  # K

    Mw = [0.02, 0.02, 0.02]  # Mulecular weight units kg/mol
    #Other Parameter
    A_bh = π.*(D_bh./2).^2
    LA_bh  = L_bh.*A_bh
    A_w = (π.*D_w.^2/4)
    LwAw  = L_w.*A_w
    g = [9.81, 9.81, 9.81] # m/s2
    R = [8.314, 8.314, 8.314] # J/mol K
    V_a = L_a.*(π.*(D_a./2).^2 - π.*(D_w./2).^2)
end