using Plots
using Parameters

# plotly()
gr()
include("Plant.jl")
include("Optimizer_N.jl")
include("Optimizer_R.jl")
include("InitiStates.jl")

## Initial Parameters
Nw = 3;
t_min_o = 2.5; t_min_p = 1; t_min_s = 15; #Duration for optimizer, Plant, and Simulation respectively
U_opt = 4*ones(Nw, 15 * t_min_p + 1); # u0
#Prices
αₚₒ = 10; α_gl = 0.1; γ = 0.1;
##Loading steady states for beginning
t_ss = 60;
U_opts = 4*ones(Nw, 15 * t_ss + 1);
Initial = Initial_States(U_opts, Nw, t_ss, t_min_p)
# Initial = Initial_Statess(Nw)

## Simulation
println("Simulation is Starting!")
	# do k=1 separately
	k=1
	U_opt, ω_po, ω_pg, obj = Plant_optimizer(U_opt, t_min_o, t_min_p, Nw, Initial, αₚₒ, α_gl, γ, false);
	Inits = Plant(Nw, U_opt, Initial, t_min_p);

	U_opt_hist   = U_opt;
	ω_po_profile = Inits[:ω_po0];
	ω_pg_profile = Inits[:ω_pg0];



	for k in 2:5
		U_opt, ω_po, ω_pg,obj  = Plant_optimizer(U_opt, t_min_o, t_min_p, Nw, Inits, αₚₒ, α_gl, γ, false);
		U_opt_hist = hcat(U_opt_hist, U_opt[:, 2:end]);
		Inits= Plant(Nw, U_opt, Inits, t_min_p);

		ω_po_profile = hcat(ω_po_profile, Inits[:ω_po0][:, 2:end]);
		ω_pg_profile = hcat(ω_pg_profile, Inits[:ω_pg0][:, 2:end]);
	end
	for k in 6:10
		U_opt, ω_po, ω_pg,obj  = Plant_optimizerr(U_opt, t_min_o, t_min_p, Nw, Inits, αₚₒ, α_gl, γ, false);
		U_opt_hist = hcat(U_opt_hist, U_opt[:, 2:end]);
		Inits= Plant(Nw, U_opt, Inits, t_min_p);

		ω_po_profile = hcat(ω_po_profile, Inits[:ω_po0][:, 2:end]);
		ω_pg_profile = hcat(ω_pg_profile, Inits[:ω_pg0][:, 2:end]);
	end
	for k in 11:15
		U_opt, ω_po, ω_pg,obj  = Plant_optimizer(U_opt, t_min_o, t_min_p, Nw, Inits, αₚₒ, α_gl, γ, false);
		U_opt_hist = hcat(U_opt_hist, U_opt[:, 2:end]);
		Inits= Plant(Nw, U_opt, Inits, t_min_p);

		ω_po_profile = hcat(ω_po_profile, Inits[:ω_po0][:, 2:end]);
		ω_pg_profile = hcat(ω_pg_profile, Inits[:ω_pg0][:, 2:end]);
	end


println("Simulation has ended!")

## plotting

plotlyjs()
gr()
t_points = collect(0:4:t_min_s*60)
Max_Gas = vec(hcat(28*ones(1, 15*5 + 1), 25*ones(1, 15 * 5 ), 28*ones(1, 15 * 5 )))
p1=plot(t_points/60,U_opt_hist', linetype=:steppre, title="gas lift",label=["well1" "well2" "well3"])
p_po=plot(t_points/60,ω_po_profile', title="Produced oil",label=false)
p_pg=plot(t_points/60,ω_pg_profile', title="Produced gas",xlabel = "Time (min)",label=false)
fig1 = plot(p1,p_po,p_pg , layout = (3, 1))
##
t_points = collect(0:4:t_min_s*60)
p1=plot(t_points/60,    sum(U_opt_hist[i,:] for i in 1:Nw) , linetype=:steppre,title="gas lift",label="All wells together",color = :bluesreds, legend = (0.4, 0.3))
p_po=plot(t_points/60,  sum(ω_po_profile[i,:] for i in 1:Nw) , linetype=:steppre,title="Total produced oil",label=false,color = :bluesreds)
p_pg=plot(t_points/60,  sum(ω_pg_profile[i,:] for i in 1:Nw) , linetype=:steppre,xlabel = "Time (min)",label=false,color = :bluesreds)
p_pg=plot!(t_points/60,  Max_Gas , linetype=:steppre,title="Total produced gas",xlabel = "Time (min)",label=" Maximum allowed gas", linestyle=:dash, legend = (0.4, 0.3))
fig2 = plot(p1,p_po,p_pg , layout = (3, 1))

fig3 =  plot(fig1, fig2 , layout = (1, 2), yaxis="[kg/s]", yguidefontsize=7)

savefig(fig3, "Figures//11Jan//Results_part4.pdf")




#p1=plot(x,[ua u],xlabel = "x", ylabel = "u", color = [:deepskyblue :red],  label = ["Exact" "Numerical"], lw = 2,linealpha=[1.0 .2],marker = ( [:none :o],0.75, Plots.stroke(:black)),markersize=2, legend=(-35.0, 1.0),framestyle = :box, grid=false)
#julia> plot(rand(40),xtickfontsize=18,ytickfontsize=18,xlabel="wavelength",xguidefontsize=18,yscale=:log10,ylabel="flux",yguidefontsize=18,legendfontsize=18) here
##Save the Figures
using Dates
a = today()
savefig(fig3, "Figures//11Jan//abcd.pdf")
## Profit Computing

profit = αₚₒ * sum(ω_po_profile) - α_gl * sum(U_opt_hist)
##Data recording center
U_opt_gmaxr = U_opt_hist
ω_po_gmaxr = ω_po_profile
ω_pg_gmaxr = ω_pg_profile
#--------------------------------
U_opt_gmaxp5 = U_opt_hist;
 ω_po_gmaxp5 = ω_po_profile;
 ω_pg_gmaxp5 = ω_pg_profile;
#--------------------------------
U_opt_gmaxm5 = U_opt_hist;
 ω_po_gmaxm5 = ω_po_profile; 
 ω_pg_gmaxm5 = ω_pg_profile;
##



p1=plot(t_points/60,U_opt_gmaxr', linetype=:steppre, title="gas lift",label=["w1 - 1.2" "w2 - 2.2" "w3 - 3.2"])
p_po=plot(t_points/60,ω_po_gmaxr', title="Produced oil",label=false)
p_pg=plot(t_points/60,ω_pg_gmaxr', title="Produced gas",xlabel = "Time (min)",label=false)
fig1 = plot(p1,p_po,p_pg , layout = (3, 1))















t_points = collect(0:4:t_min_s*60)

p1=plot(t_points/60,    sum(U_opt_gmaxr[i,:] for i in 1:Nw) , linetype=:steppre,title="Gas lift",label="PI=[1.2, 2.2, 3.2]",color = :bluesreds, labelsize=50)
p1=plot!(t_points/60,    sum(U_opt_gmaxp5[i,:] for i in 1:Nw) , linetype=:steppre,title="Gas lift",label="PI=[1.26, 2.31, 3.36]",color = :red, labelsize=50)
# p1=plot!(t_points/60,    sum(U_opt_gmaxp5[i,:] for i in 1:Nw) , linetype=:steppre,title="gas lift",label="PI=[1.14, 2.09, 3.04]",color = :green)

p_po=plot(t_points/60,  sum(ω_po_gmaxr[i,:] for i in 1:Nw) , linetype=:steppre,title="Total produced oil",label=false,color = :bluesreds)
p_po=plot!(t_points/60,  sum(ω_pg_gmaxp5[i,:] for i in 1:Nw) , linetype=:steppre,title="Total produced oil",label=false,color = :red)
# p_po=plot!(t_points/60,  sum(ω_pg_gmaxp5[i,:] for i in 1:Nw) , linetype=:steppre,title="Total produced oil",label=false,color = :green)

p_pg=plot(t_points/60,  sum(ω_pg_profile[i,:] for i in 1:Nw) , linetype=:steppre,xlabel = "Time (min)",label=false,color = :bluesreds)
p_pg=plot!(t_points/60,  sum(ω_pg_gmaxp5[i,:] for i in 1:Nw) , linetype=:steppre,xlabel = "Time (min)",label=false,color = :red)
# p_pg=plot!(t_points/60,  sum(ω_pg_gmaxp5[i,:] for i in 1:Nw) , linetype=:steppre,xlabel = "Time (min)",label=false,color = :green)

p_pg=plot!(t_points/60,  28 * ones(length(t_points)) , linetype=:steppre,title="Total produced gas",xlabel = "Time (min)",label=" Maximum allowed gas", linestyle=:dash)
fig4 = plot(p1,p_po,p_pg , layout = (3, 1), legend = (0.7, -.1), yaxis="[kg/s]")


savefig(fig4, "Figures//11Jan//Results_part3.pdf")


profitr = αₚₒ * sum(ω_po_gmaxr) - α_gl * sum(U_opt_gmaxr)
profitp = αₚₒ * sum(ω_pg_gmaxm5) - α_gl * sum(U_opt_gmaxm5)